//+------------------------------------------------------------------+
//|                                                  LoadHistory.mqh |
//|                        Copyright 2018, MetaQuotes Software Corp. |
//|                         https://www.mql5.com/ru/users/nikolay7ko |
//+------------------------------------------------------------------+
#property copyright "Copyright 2018, Nikolay Semko"
#property link      "https://www.mql5.com/ru/users/nikolay7ko"
#property link      "SemkoNV@bk.ru"  
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CLoadHistory
  {
private:
   int               N;
   bool              sync;
   string            symbol;
   ENUM_TIMEFRAMES   TF;
   datetime          FirstDate;
   int               MaxBar;
   int               bars;
   int               PerSec;

public:
                     CLoadHistory(string Symb="",ENUM_TIMEFRAMES tf=0,int n=0);
                    ~CLoadHistory();
   double            open[];
   double            close[];
   double            low[];
   double            high[];
   datetime          time[];
   void              NewTick();
   void              Timer();
   int               BarShift(datetime t);
   int               BarShift(datetime t,datetime &arr[],ENUM_TIMEFRAMES tf);
   int               Size() const {return(N);}
   bool              Sync() const {return(sync);}
   double            Percentage();
   string            Symb() { return(symbol);}
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CLoadHistory::CLoadHistory(string Symb="",ENUM_TIMEFRAMES tf=0,int n=0)
  {
   if(Symb=="") symbol=_Symbol; else symbol=Symb;
   if(tf==0) TF=_Period; else TF=tf;
   sync=false;
   if(TF!=PERIOD_MN1) PerSec=PeriodSeconds(TF); else PerSec=2678400;
   FirstDate=(datetime)SeriesInfoInteger(symbol,TF,SERIES_SERVER_FIRSTDATE);
   MaxBar=TerminalInfoInteger(TERMINAL_MAXBARS);
   N=(int)fmin(MaxBar,n);
   ArrayResize(time,N);
   EventSetMillisecondTimer(1000);
   Timer();
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CLoadHistory::~CLoadHistory()
  {
  }
//+------------------------------------------------------------------+
void CLoadHistory::Timer()
  {
   if(!sync)
     {
      bars=CopyTime(symbol,TF,0,N,time);
      Comment("Loading of history: "+DoubleToString(Percentage(),2)+" %");
      if(bars>=N || ((time[0]-time[0]%PeriodSeconds(PERIOD_D1))<=FirstDate && time[0]>0))
        {
         N=bars;
         ArrayResize(time,N);
         ArrayResize(close,N);
         ArrayResize(open,N);
         ArrayResize(low,N);
         ArrayResize(high,N);

         if(CopyClose(symbol,TF,0,N,close)==N)
            if(CopyOpen(symbol,TF,0,N,open)==N)
               if(CopyLow(symbol,TF,0,N,low)==N)
                  if(CopyHigh(symbol,TF,0,N,high)==N)
                    {
                     if(symbol==_Symbol) EventKillTimer();
                     else EventSetMillisecondTimer(100);
                     sync=true;
                     Comment("");
                    }
        }
     }
   else NewTick();
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void   CLoadHistory::NewTick()
  {
   if(!sync) {Timer(); return;}
   MqlRates R[];
   if(N<1 || CopyRates(symbol,TF,0,1,R)<=0) return;
   int delta=int(R[0].time-time[N-1]);
   if(delta)
     {
      if(delta<=PerSec)
        {
         N++;
         ArrayResize(time,N);
         ArrayResize(close,N);
         ArrayResize(open,N);
         ArrayResize(low,N);
         ArrayResize(high,N);
         open[N-1]=R[0].open;
         time[N-1]=R[0].time;
        }
      else
        {
         sync=false;
         EventSetMillisecondTimer(100);
         Timer();
         return;
        }
     }
   close[N-1]=R[0].close;
   low[N-1]=R[0].low;
   high[N-1]=R[0].high;
  }
//+------------------------------------------------------------------+
double   CLoadHistory::Percentage()
  {
   static double prePer=0;
   if(bars<=0) return(prePer);
   double Per=100.0*(TimeCurrent()-time[0])/(TimeCurrent()-FirstDate);
   if(Per<prePer) Per=prePer;
   prePer=Per;
   return(Per);
  }
//+------------------------------------------------------------------+
int CLoadHistory::BarShift(datetime t)
  {
   int cur,Start=0;
   int fin=N-1;
   if(t<time[0] || t>=time[fin]) return(0);
   if(t>=time[fin] && t<(time[fin]+PerSec)) return(fin);
   while(true)
     {
      cur=(Start+fin)/2;
      if(t<time[cur]) {if(t>=time[cur-1]) return(cur-1);    fin=cur;}
      else           {if(t<time[cur+1]) return(cur);Start=cur;}
      if((time[fin]-time[Start])==((fin-Start)*PerSec) && TF!=PERIOD_MN1)
         return(Start+int(int(t-time[Start])/PerSec));
     }
  }
//+------------------------------------------------------------------+

int CLoadHistory::BarShift(datetime t,datetime &arr[],ENUM_TIMEFRAMES tf)
  {
   int size=ArraySize(arr);
   int PrSc=PeriodSeconds(tf);
   int cur,Start=0;
   int fin=size-1;
   if(t<arr[0] || t>=arr[fin]) return(0);
   if(t>=arr[fin] && t<(arr[fin]+PrSc)) return(fin);
   while(true)
     {
      cur=(Start+fin)/2;
      if(t<arr[cur]) {if(t>=arr[cur-1]) return(size-cur);    fin=cur;}
      else           {if(t< arr[cur+1]) return(size-cur-1);Start=cur;}
      if((arr[fin]-arr[Start])==((fin-Start)*PrSc) && tf!=PERIOD_MN1)
         return(size-1-Start-int(int(t-arr[Start])/PrSc));
     }
  }